@@ -1,7 +1,7 @@
 /*
-   engn/perldb2/dbdimp.c, engn_perldb2, db2_v3, 1.3 98/09/24 16:20:14
+   engn/perldb2/dbdimp.c, engn_perldb2, db2_v6, 1.3 99/01/12 14:27:40
 
-   Copyright (c) 1995,1996,1997,1998 International Business Machines Corp.
+   Copyright (c) 1995,1996,1997,1998,1999 International Business Machines Corp.
 */
 
 #include <stdio.h>
@@ -164,7 +164,8 @@ dbd_db_connect(dbh,imp_dbh,dbname,uid,pwd)
     char *msg;
     SQLINTEGER ret;
     
-    ret = SQLAllocConnect(imp_drh->henv,&imp_dbh->hdbc);
+    ret = SQLAllocHandle(SQL_HANDLE_DBC, imp_drh->henv, 
+                         &imp_dbh->hdbc);                         
     msg = (ERRTYPE(ret) ? "Connect allocation failed" :
                                 "Invalid Handle");
     ret = check_error(dbh,ret,msg);
@@ -191,8 +192,9 @@ dbd_db_connect(dbh,imp_dbh,dbname,uid,pwd)
         return(ret);            /* Must return SQL codes not perl/DBD/DBI */
     }                             /* otherwise failure is not caught  */
     /* DBI spec requires AutoCommit on */
-    ret = SQLSetConnectOption(imp_dbh->hdbc, SQL_AUTOCOMMIT, SQL_AUTOCOMMIT_ON);
-    msg = (ERRTYPE(ret) ? "SetConnectOption Failed" : "Invaild handle");
+    ret = SQLSetConnectAttr(imp_dbh->hdbc, SQL_ATTR_AUTOCOMMIT,   
+                            (void *)SQL_AUTOCOMMIT_ON, 0);        
+    msg = (ERRTYPE(ret) ? "SetConnectAttr Failed" : "Invalid handle");
     ret = check_error(dbh,ret,msg);
     if (ret != SQL_SUCCESS) {
         SQLFreeConnect(imp_dbh->hdbc);
@@ -221,7 +223,8 @@ dbd_db_login(dbh, imp_dbh, dbname, uid, pwd)
     char *msg;
 
     if (! imp_drh->connects) {
-        ret = SQLAllocEnv(&imp_drh->henv);
+        ret = SQLAllocHandle(SQL_HANDLE_ENV, SQL_NULL_HANDLE,     
+	                     &imp_drh->henv);                     
         msg = (imp_drh->henv == NHENV ?
                 "Total Environment allocation failure!" :
                 "Environment allocation failed");
@@ -259,7 +262,8 @@ dbd_db_do(dbh, statement) /* error : <=(-2), ok row count : >=0, unknown count :
     char *msg;
     SQLHSTMT stmt;
 
-    ret = SQLAllocStmt(imp_dbh->hdbc, &stmt);
+    ret = SQLAllocHandle(SQL_HANDLE_STMT, imp_dbh->hdbc,          
+                         &stmt);                                  
     msg = "Statement allocation error";
     ret = check_error(dbh, ret, msg);
     if (ret < 0)                                                  
@@ -386,8 +390,10 @@ dbd_db_STORE(dbh, keysv, valuesv)
     char *msg;
     
     if (kl==10 && strEQ(key, "AutoCommit")) {
-        ret = SQLSetConnectOption(imp_dbh->hdbc,SQL_AUTOCOMMIT,
-                (on ? SQL_AUTOCOMMIT_ON : SQL_AUTOCOMMIT_OFF ));
+	ret = SQLSetConnectAttr(imp_dbh->hdbc,                   
+	                        SQL_ATTR_AUTOCOMMIT,
+              (on ? (void *)SQL_AUTOCOMMIT_ON : (void *)SQL_AUTOCOMMIT_OFF),
+		                0);                              
         if ( ret ) {
             msg = (ERRTYPE(ret) ? "Change of AUTOCOMMIT failed" :
                                   "Invalid Handle");
@@ -447,7 +453,8 @@ dbd_st_prepare(sth, statement, attribs)
 
     imp_sth->done_desc = 0;
 
-    ret = SQLAllocStmt(imp_dbh->hdbc,&imp_sth->phstmt);
+    ret = SQLAllocHandle(SQL_HANDLE_STMT, imp_dbh->hdbc,          
+                         &imp_sth->phstmt);                       
     msg = "Statement allocation error";
     ret = check_error(sth,ret,msg);
 
@@ -459,6 +466,10 @@ dbd_st_prepare(sth, statement, attribs)
 
     EOI(ret);
 
+    if (dbis->debug >= 2)                                        
+        fprintf(DBILOGFP, "    dbd_st_prepare'd sql f%d\n\t%s\n",
+	        imp_sth->phstmt, statement);                     
+
     ret = SQLNumParams(imp_sth->phstmt,&params);
     msg = "Unable to determine number of parameters";
     ret = check_error(sth,ret,msg);
@@ -475,6 +486,7 @@ dbd_st_prepare(sth, statement, attribs)
 
     /* initialize sth pointers */
     imp_sth->RowCount = -1;                                       
+	imp_sth->has_inouts = 0;
 
     DBIc_IMPSET_on(imp_sth);
     return 1;
@@ -540,6 +552,7 @@ dbd_preparse(imp_sth, statement)
     if (imp_sth->bind_names == NULL)
         imp_sth->bind_names = newHV();
     phs_tpl.sv = &sv_undef;
+	phs_tpl.is_inout = 0;
     phs_sv = newSVpv((char *)&phs_tpl, sizeof(phs_tpl));
     hv_store(imp_sth->bind_names, (char *)start, 
                              (STRLEN)(dest-start), phs_sv, 0);
@@ -557,7 +570,7 @@ dbd_preparse(imp_sth, statement)
 int
 dbd_bind_ph(sth, ph_namesv, newvalue, attribs)
     SV *sth;
-    SV *ph_namesv;
+    SV *ph_namesv;		/* index of parameter 1..n */
     SV *newvalue;
     SV *attribs;
 {
@@ -592,11 +605,10 @@ dbd_bind_ph(sth, ph_namesv, newvalue, attribs)
 
     svp = hv_fetch(imp_sth->bind_names, (char *)name, name_len, 0);
     if (svp == NULL)
-        croak("dbd_bind_ph placeholder '%s' unknown", name);
+        croak("Can't bind unknown parameter marker '%s'", name);  
     phs = (phs_t*)((void*)SvPVX(*svp));        /* placeholder struct    */
 
     if (phs->sv == &sv_undef) {     /* first bind for this placeholder    */
-    phs->sv = newSV(0);
     phs->ftype = 1;
     }
 
@@ -617,7 +629,7 @@ dbd_bind_ph(sth, ph_namesv, newvalue, attribs)
             nullok = SvIV(*svp);
 
 
-    }    /* else if NULL / UNDEF then don't alter attributes.    */
+    } /* else if NULL / UNDEF then default to values assigned at top */
     /* This approach allows maximum performance when    */
     /* rebinding parameters often (for multiple executes).    */
 
@@ -625,26 +637,74 @@ dbd_bind_ph(sth, ph_namesv, newvalue, attribs)
     /* Later we may optimise this so that more often we can    */
     /* just copy the value & length over and not rebind.    */
 
-    if (SvOK(newvalue)) {
-        /* XXX need to consider oraperl null vs space issues?    */
-        /* XXX need to consider taking reference to source var    */
-        sv_setsv(phs->sv, newvalue);
-        value_ptr = SvPV(phs->sv, value_len);
-        phs->indp = 0;
-    } else {
-        sv_setsv(phs->sv,0); 
-        value_ptr = "";
-        value_len = 0;
-        phs->indp = SQL_NULL_DATA;
-    }
+	{
+	  bool is_inout = !(param == SQL_PARAM_INPUT);
+
+	  if ((!is_inout && phs->is_inout) ||
+		  (is_inout && !phs->is_inout && phs->sv != &sv_undef)) {
+		(void)SvREFCNT_dec(phs->sv);
+		phs->sv = &sv_undef;
+	  }
+	  phs->is_inout = is_inout;
+
+	  if (phs->sv == &sv_undef && !is_inout) {     /* first bind for this placeholder    */
+		phs->sv = newSV(0);
+	  }
+
+	  if (is_inout) {
+		int newlen;
+
+		imp_sth->has_inouts = 1;
+
+        if (SvREADONLY(newvalue))
+		  croak("Cannot modify readonly SV");
+        (void)SvUPGRADE(newvalue, SVt_PVNV);
+		newlen = (prec < 28) ? 28 : prec+1;
+        SvGROW(newvalue, newlen);
+
+		if (SvOK(newvalue)) {
+		  phs->sv = newvalue;
+		  SvREFCNT_inc(phs->sv);
+		  value_ptr = SvPV(phs->sv, value_len);
+		  value_len = newlen;
+		  phs->indp = 0;
+		} else {
+		  croak("Can't bind non-scalar in inout position");
+		}
+	  } else {
+		if (SvOK(newvalue)) {
+		  sv_setsv(phs->sv, newvalue);
+		  value_ptr = SvPV(phs->sv, value_len);
+		  phs->indp = 0;
+		} else {
+		  sv_setsv(phs->sv,0); 
+		  value_ptr = "";
+		  value_len = 0;
+		  phs->indp = SQL_NULL_DATA;
+		}
+	  }
+	}
 
     if (!nullok && !SvOK(phs->sv)) {
         fprintf(stderr,"phs->sv is not OK\n");
     }    
+
+    if (dbis->debug >= 2)                                        
+        fprintf(DBILOGFP,
+	        "  bind %s: ParmType=%d, Ctype=%d, SQLtype=%d, Prec=%d, Scale=%d\n",
+		name, param, ctype, stype, prec, scale);         
+
+	{
+	  char *cptr = (phs->indp == 0)?SvPVX(phs->sv):NULL;
+
     ret = SQLBindParameter(imp_sth->phstmt,(SQLINTEGER)SvIV(ph_namesv),
-                           param,ctype,stype,prec,scale,
-                           (phs->indp == 0)?SvPVX(phs->sv):NULL,  
-	                   0, (phs->indp != 0 && nullok)?&phs->indp:NULL);
+                           param,ctype,stype,
+						   (prec > 0 ? prec : value_len),
+						   scale,  
+                           cptr,  
+						   value_len,                             
+	                   (phs->is_inout || (phs->indp != 0 && nullok))?&phs->indp:NULL);
+	}
     msg = ( ERRTYPE(ret) ? "Bind failed" : "Invalid Handle");
     ret = check_error(sth,ret,msg);
     EOI(ret);
@@ -684,8 +744,7 @@ dbd_describe(h, imp_sth)
 
     /* Get number of fields and space needed for field names    */
     for(i=0; i < num_fields; ++i ) {
-        SQLCHAR  cbuf[MAX_COL_NAME_LEN];
-        SQLCHAR  dbtype;
+        SQLCHAR  cbuf[MAX_COL_NAME_LEN+1];                        
         imp_fbh_t *fbh = &imp_sth->fbh[i];
         f_cbufl[i] = sizeof(cbuf);
 
@@ -694,14 +753,16 @@ dbd_describe(h, imp_sth)
         msg    = (ERRTYPE(ret) ? "DescribeCol failed" : "Invalid Handle");
         ret = check_error(h,ret,msg);
         EOI(ret);
-        ret = SQLColAttributes(imp_sth->phstmt,i+1,SQL_COLUMN_LENGTH,
-                NULL, 0, NULL ,&fbh->dbsize);
-        msg    = (ERRTYPE(ret) ? "ColAttributes failed" : "Invalid Handle");
+        ret = SQLColAttribute(imp_sth->phstmt,i+1,                
+	                      SQL_DESC_OCTET_LENGTH,              
+                              NULL, 0, NULL ,&fbh->dbsize);
+        msg    = (ERRTYPE(ret) ? "ColAttribute failed" : "Invalid Handle");
         ret = check_error(h,ret,msg);
         EOI(ret);
-        ret = SQLColAttributes(imp_sth->phstmt,i+1,SQL_COLUMN_DISPLAY_SIZE,
-                NULL, 0, NULL ,&fbh->dsize);
-        msg    = (ERRTYPE(ret) ? "ColAttributes failed" : "Invalid Handle");
+        ret = SQLColAttribute(imp_sth->phstmt,i+1,                
+	                      SQL_DESC_DISPLAY_SIZE,              
+                              NULL, 0, NULL ,&fbh->dsize);
+        msg    = (ERRTYPE(ret) ? "ColAttribute failed" : "Invalid Handle");
         ret = check_error(h,ret,msg);
         EOI(ret);
 
@@ -786,17 +847,43 @@ dbd_st_execute(sth)	/* error : <=(-2), ok row count : >=0, unknown count : (-1)
     char *msg;
     SQLINTEGER ret;
  
-    /* describe and allocate storage for results        */
-    if (!imp_sth->done_desc && !dbd_describe(sth, imp_sth)) {
-        /* dbd_describe has already called check_error()        */
-        return -2;                                                
-    }
     ret = SQLExecute(imp_sth->phstmt);
     msg = "SQLExecute failed";
     ret = check_error(sth,ret,msg);
     if (ret < 0)                                                 
     	return(-2);
 
+    if (imp_sth->bind_names && imp_sth->has_inouts) {
+	  HV *hv = imp_sth->bind_names;
+	  SV *sv;
+	  char *key;
+	  I32 retlen;
+	  hv_iterinit(hv);
+	  while( (sv = hv_iternextsv(hv, &key, &retlen)) != NULL ) {
+        phs_t *phs;
+        if (sv != &sv_undef) {
+		  phs = (phs_t*)SvPVX(sv);
+		  if (phs->is_inout) {
+			if (phs->indp == SQL_NULL_DATA) {
+			  SvCUR_set(phs->sv,0);
+			} else if (phs->indp == SQL_NO_TOTAL) {
+			  warn("inout parameter length was not determinable after statement execution");
+			} else if (phs->indp <= SvLEN(phs->sv)) {
+			  SvCUR_set(phs->sv,phs->indp);
+			} else {
+			  warn("inout parameter length was too long -- possible memory corruption");
+			}
+		  }
+        }
+	  }
+    }
+
+    /* describe and allocate storage for results        */
+    if (!imp_sth->done_desc && !dbd_describe(sth, imp_sth)) {
+        /* dbd_describe has already called check_error()        */
+        return -2;                                                
+    }
+
     ret = SQLRowCount(imp_sth->phstmt, &imp_sth->RowCount);
     msg = "SQLRowCount failed";
     ret = check_error(sth, ret, msg);                            
@@ -827,7 +914,13 @@ dbd_st_fetch(sth)
         check_error(sth, -3,"no statement executing");
         return Nullav;
     }
-    
+
+	/*
+	if (imp_sth->RowCount == -1) {
+	  warn("no row-set from preceding statement");
+	  return Nullav;
+	}
+    */
     ret = SQLFetch(imp_sth->phstmt);
     if (ret != SQL_SUCCESS && ret != SQL_SUCCESS_WITH_INFO) {
         if (ret != SQL_NO_DATA_FOUND) {    /* was not just end-of-fetch    */
